@echo off
REM *************************************
REM *** Change these input-parameters ***
REM *************************************
set opening_plies=16
set games=ccc-finals-16.pgn
set reference_eng="Stockfish 211115"
set /A reference_elo=3780

REM **********************************************
REM *** Start of the batch-tool, do not change ***
REM **********************************************

REM *** Cleanup and change the working path to the bin-folder ***
del Gamepairs_final.pgn 2>nul
del Gamepairs_head-to-head.txt 2>nul
del Gamepairs_rating.txt 2>nul
cd ./bin
del box*.pgn 2>nul
del Gamepairs_final.pgn 2>nul
del cut.pgn 2>nul
del winwhite.pgn 2>nul
del winblack.pgn 2>nul
del losswhite.pgn 2>nul
del lossblack.pgn 2>nul
del draw.pgn 2>nul
del unique_draw.pgn 2>nul
del won_pairs.pgn 2>nul
del loss_pairs.pgn 2>nul
del draw_pairs.pgn 2>nul
del work01.pgn 2>nul
del work02.pgn 2>nul
del 20_00_eng1.pgn 2>nul
del 15_05_eng1.pgn 2>nul
del 15_05_eng1_A.pgn 2>nul
del 15_05_eng1_B.pgn 2>nul
del 10_10_eng1_A.pgn 2>nul
del 10_10_eng1_B.pgn 2>nul
del 10_10_eng1_C.pgn 2>nul
del 10_10_eng1_D.pgn 2>nul
del 10_10_eng1_E.pgn 2>nul
del 10_10_eng1_F.pgn 2>nul
del 05_15_eng1.pgn 2>nul
del 05_15_eng1_A.pgn 2>nul
del 05_15_eng1_B.pgn 2>nul
del 00_20_eng1.pgn 2>nul
del final_pairs.pgn 2>nul


@echo ************************************************************
@echo ***     Gamepair Rescorer V1.0 by Stefan Pohl (SPCC)     ***
@echo ***              (C) 2021, www.sp-cc.de                  ***
@echo ************************************************************
@echo ************************************************************
@echo ************************************************************

REM *** Cut all games after the opening-moves, and delete comments and useless tags ***
@echo Step 1: Cut all games after the opening-moves and clean pgn
pgn-extract --quiet -C -N -V --seven --plylimit %opening_plies% ../%games% --output cut.pgn

REM *** Split the new shortened gamefile into head-to-head files ***
@echo ************************************************************
@echo Step 2: Split gamefile into head-to-head files
pairSplit cut.pgn >nul

REM *** Count, how many files pairSplit made *** 
set /A "filenumber=0"
for /F "delims=" %%A IN (manifest-box) do set /A "filenumber+=1"
set /A show_fn=%filenumber%
set /A filenumber=%filenumber%-1
@echo Number of head-to-head files generated: %show_fn%
@echo ************************************************************

REM *** Check, if the number of head-to-files is not too big ***
if %show_fn% GEQ 996 (
@echo ERROR: Overflow! Too much head-to-head files
@echo Program terminated...
@echo ************************************************************
set /A overflow=1
goto exitloop01
) else (
set /A overflow=0
)

REM *** Start working on each head-to-head file, sequentially ***
@echo Step 3: Start processing head-to head files sequentially
set /A filecounter=-1
:loop01
set /A filecounter=%filecounter%+1
set /A show_fc=%filecounter%+1

@echo Processing head-to-head file %show_fc% of %show_fn%

REM *** Build the filename (boxXXX.pgn) for the next file ***
if %filecounter% GEQ 100 set filename=box%filecounter%.pgn
if %filecounter% LEQ 99 set filename=box0%filecounter%.pgn
if %filecounter% LEQ 9 set filename=box00%filecounter%.pgn

REM *** Extract one of the two engine-names for further processing ***
nameList %filename% >nul
set /p engine1=<outName

REM **************************************************************
REM *** Do the gamepairing operations on the head-to-head-file ***
REM **************************************************************

REM *** Find won, lost and draw games (white and black) (from the point of view of one engine) ***
pgn-extract --quiet -Tw"%engine1%" -Tr1-0 %filename% --output winwhite.pgn
pgn-extract --quiet -Tb"%engine1%" -Tr0-1 %filename% --output winblack.pgn
pgn-extract --quiet -Tb"%engine1%" -Tr1-0 %filename% --output lossblack.pgn
pgn-extract --quiet -Tw"%engine1%" -Tr0-1 %filename% --output losswhite.pgn
pgn-extract --quiet -Tr1/2-1/2 %filename% --output draw.pgn

pgn-extract --quiet --fuzzydepth 0 -D draw.pgn --output unique_draw.pgn

REM *** Build the 5 possible results-gamebases (2-0, 1.5-0.5, 1.0-1.0, 0.5-1.5, 0-2) ***
pgn-extract --quiet -U -d20_00_eng1.pgn winwhite.pgn winblack.pgn 

pgn-extract --quiet -U -d10_10_eng1_A.pgn draw.pgn 
pgn-extract --quiet -U -d10_10_eng1_B.pgn winwhite.pgn lossblack.pgn
pgn-extract --quiet -U -d10_10_eng1_C.pgn losswhite.pgn winblack.pgn
pgn-extract --quiet -o10_10_eng1_D.pgn 10_10_eng1_A.pgn 10_10_eng1_B.pgn
pgn-extract --quiet -o10_10_eng1_E.pgn 10_10_eng1_C.pgn 10_10_eng1_D.pgn
textReplace 10_10_eng1_E.pgn 10_10_eng1_F.pgn "1-0" "1/2-1/2"
textReplace 10_10_eng1_F.pgn draw_pairs.pgn "0-1" "1/2-1/2"

pgn-extract --quiet -U -d15_05_eng1_A.pgn unique_draw.pgn winwhite.pgn 
pgn-extract --quiet -U -d15_05_eng1_B.pgn unique_draw.pgn winblack.pgn 
pgn-extract --quiet -o15_05_eng1.pgn 15_05_eng1_A.pgn 15_05_eng1_B.pgn

pgn-extract --quiet -U -d05_15_eng1_A.pgn unique_draw.pgn losswhite.pgn
pgn-extract --quiet -U -d05_15_eng1_B.pgn unique_draw.pgn lossblack.pgn
pgn-extract --quiet -o05_15_eng1.pgn 05_15_eng1_A.pgn 05_15_eng1_B.pgn

pgn-extract --quiet -U -d00_20_eng1.pgn losswhite.pgn lossblack.pgn 

REM *** Merge (2-0 and 1.5-0.5) and (0-2 and 0.5-1.5) gamebases into one file***
pgn-extract --quiet -owon_pairs.pgn 20_00_eng1.pgn 15_05_eng1.pgn
pgn-extract --quiet -oloss_pairs.pgn 00_20_eng1.pgn 05_15_eng1.pgn

REM *** Merge all 3 remaining gamepairs (W-D-L) into one file ***
pgn-extract --quiet -owork01.pgn won_pairs.pgn draw_pairs.pgn
pgn-extract --quiet -owork02.pgn work01.pgn loss_pairs.pgn 
pgn-extract --quiet -C -N -V work02.pgn --output final_pairs.pgn

REM **************************************************************

REM *** collect the gamepairs-data in one pgn-file (Gamepairs_final.pgn) ***
pgn-extract --quiet -aGamepairs_final.pgn final_pairs.pgn

REM *** delete the processed head-to-head file, which is no longer needed ***
del %filename%

REM *** All head-to-head files done? Then exit, otherwise jump back and proceed ***
if %filecounter% GEQ %filenumber% goto exitloop01
goto loop01
:exitloop01

REM *** Cleanup ***
del cut.pgn 2>nul
del box*.pgn 2>nul
del winwhite.pgn 2>nul
del winblack.pgn 2>nul
del losswhite.pgn 2>nul
del lossblack.pgn 2>nul
del draw.pgn 2>nul
del unique_draw.pgn 2>nul
del won_pairs.pgn 2>nul
del loss_pairs.pgn 2>nul
del draw_pairs.pgn 2>nul
del work01.pgn 2>nul
del work02.pgn 2>nul
del 20_00_eng1.pgn 2>nul
del 15_05_eng1.pgn 2>nul
del 15_05_eng1_A.pgn 2>nul
del 15_05_eng1_B.pgn 2>nul
del 10_10_eng1_A.pgn 2>nul
del 10_10_eng1_B.pgn 2>nul
del 10_10_eng1_C.pgn 2>nul
del 10_10_eng1_D.pgn 2>nul
del 10_10_eng1_E.pgn 2>nul
del 10_10_eng1_F.pgn 2>nul
del 05_15_eng1.pgn 2>nul
del 05_15_eng1_A.pgn 2>nul
del 05_15_eng1_B.pgn 2>nul
del 00_20_eng1.pgn 2>nul
del final_pairs.pgn 2>nul
del excludeBox.pgn 2>nul
del manifest-box 2>nul
del outName 2>nul
del outName2 2>nul
del outName3 2>nul

REM *** If an overflow occured, it makes no sense to start ORDO ***
if %overflow% GEQ 1 goto exit

@echo ************************************************************
@echo Step 4: Elo-calculating with ORDO

ordo-win64 -n 1 -J -j "../Gamepairs_head-to-head.txt" -q -N 0,1 -U "0,1,2,4,7,8,9,5,6" -a %reference_elo% -A %reference_eng% -o "../Gamepairs_rating.txt" -p Gamepairs_final.pgn -s500 >nul
move Gamepairs_final.pgn "../Gamepairs_final.pgn" >nul
cd ../

@echo ************************************************************
type Gamepairs_rating.txt
@echo ************************************************************
:exit
@pause